<?php
namespace Ak\Wp\Model\Entity;

use WP_Post;

/**
 * Dekorator za WordPress post.
 *
 * @package Ak\Wp\Model\Entity
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp-model
 *
 * @property int $ID
 */
class Post implements PostInterface
{
    /** @var WP_Post */
    private $post;

    /**
     * Konstruktor.
     *
     * @param WP_Post $post
     */
    public function __construct(WP_Post $post)
    {
        $this->post = $post;
    }

    /**
     * Vraća ID entiteta.
     *
     * @return null|int
     */
    public function getId(): ?int
    {
        return $this->post->ID;
    }

    /**
     * Vraća sadržaj posta.
     *
     * @return string
     */
    public function getContent(): string
    {
        return apply_filters('the_content', $this->post->post_content);
    }


    /**
     * Dohvaća instancu posta s datim ID-om.
     *
     * @param int $post_id
     *
     * @return Post
     */
    public static function get_instance(int $post_id): Post
    {
        $post = WP_Post::get_instance($post_id);
        return new self($post);
    }

    /**
     * @param string $name
     * @return bool
     */
    public function __isset(string $name)
    {
        return isset($this->post[$name]);
    }

    /**
     * @param string $name
     * @return mixed
     */
    public function __get(string $name)
    {
        return $this->post->$name;
    }

    /**
     * @param string $name
     * @param $value
     */
    public function __set(string $name, $value)
    {
        $this->post->$name = $value;
    }

    /**
     * @param string $name
     * @param array $arguments
     * @return mixed
     */
    public function __call(string $name, array $arguments)
    {
        return call_user_func_array([$this->post, $name], $arguments);
    }
}
