<?php
namespace Ak\Wp\Model\Repository;

use Ak\Orm\Entity\EntityInterface;
use Ak\Orm\Repository\RepositoryInterface;
use Ak\Wp\Model\Entity\Attachment;
use Ak\Wp\Model\Entity\Image;
use Ak\Wp\Model\Entity\Post;
use Ak\Wp\Model\Entity\PostInterface;
use Exception;
use WP_Post;

/**
 * Class PostRepository
 *
 * @package Ak\Wp\Model\Repository
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp-model
 */
class PostRepository implements RepositoryInterface
{
    /**
     * Traži post po ID-u.
     *
     * @param int $id
     *
     * @return PostInterface|null
     */
    public function find(int $id): ?EntityInterface
    {
        $post = get_post($id);
        return $post ? $this->hydratePost($post) : null;
    }

    /**
     * Traži post po slugu.
     *
     * @param string $slug
     *
     * @param string $post_type
     * @return PostInterface|null
     */
    public function findBySlug(string $slug, string $post_type = 'post'): ?PostInterface
    {
        $posts = get_posts([
            'name' => $slug,
            'post_type' => $post_type,
            'numberposts' => 1,
        ]);

        if (0 === count($posts)) {
            return null;
        }

        return $this->hydratePost($posts[0]);
    }

    /**
     * Instancira odgovarajući dekorator za dati post.
     *
     * @param WP_Post $post
     * @return PostInterface
     */
    private function hydratePost(WP_Post $post): PostInterface
    {
        switch ($post->post_type) {
            case 'attachment':
                if (preg_match('#^image/#', $post->post_mime_type)) {
                    return new Image($post);
                }

                return new Attachment($post);
            case 'post':
            default:
                return new Post($post);
        }
    }

    /** @noinspection PhpDocMissingThrowsInspection */

    /**
     * Sprema novi entitet u bazu i vraća ga.
     *
     * @param EntityInterface $entity
     * @return EntityInterface
     */
    public function insert(EntityInterface $entity): EntityInterface
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        throw new Exception('Not implemented');
    }

    /** @noinspection PhpDocMissingThrowsInspection */

    /**
     * Ažurira postojeći entitet u bazi.
     *
     * @param EntityInterface $entity
     */
    public function update(EntityInterface $entity): void
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        throw new Exception('Not implemented');
    }

    /** @noinspection PhpDocMissingThrowsInspection */

    /**
     * Briše entitet iz baze.
     *
     * @param EntityInterface $entity
     */
    public function delete(EntityInterface $entity): void
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        throw new Exception('Not implemented');
    }
}
